var matchdep = require('matchdep');

module.exports = function(grunt) {

  // load all grunt plugins from node_modules folder
  matchdep.filterAll('grunt-*').forEach(grunt.loadNpmTasks);

  grunt.initConfig({
    pkg: grunt.file.readJSON('package.json'),
    concat:{
      qunit:{
        src:['nucleo-amd.browser.js', 'tests/integration/main.js', 'tests/**/*.spec.js'],
        dest:'tests/setup/qunit/tests.js'
      },
      browser: {
        options: {
          banner: "(function(mainContext) {\n",
          footer: "\n mainContext.AMD = AMD; })(this);"
        },
        src: 'lib/main.js',
        dest:'nucleo-amd.browser.js'
      },
      nodejs: {
        options: {
          footer: "\n module.exports = AMD;"
        },
        src: 'lib/main.js',
        dest:'nucleo-amd.nodejs.js'
      }
    },
    uglify: {
        options: {
            mangle:     true,
            compress:   true,
            banner:     '/* <%= pkg.name %> v<%= pkg.version %>  */'
        },
        prod:    {
            files: {
                'nucleo-amd.min.js': ['nucleo-amd.browser.js']
            }
        }
    },
    express: {
        dev: {
            options: {
                script: 'tests/setup/server/server.js'
            }
        }
    },
    shell: {
      pre_commit_tests: {
        command: 'testem ci'
      }
    },
    watch: {
        options: {
            livereload: true,
            nospawn:    true
        },
        concat: {
            files: [
                'lib/main.js',
                'tests/integration/**/*.js'
            ],
            tasks: ['concat:browser', 'concat:qunit']
        },
    },
    bump: {
      options: {
        files: ['package.json', 'bower.json'],
        updateConfigs: ['pkg'],
        commit: true,
        commitMessage: 'Release v%VERSION%',
        commitFiles: ['-a'],
        createTag: true,
        tagName: '%VERSION%',
        tagMessage: '%VERSION%',
        push: true,
        pushTo: 'origin',
        gitDescribeOptions: '--tags --always --abbrev=1 --dirty=-d',
        globalReplace: true,
        prereleaseName: false,
        regExp: false
      }
    }

  });

// TODOS:
// - include a task to automaticaly create changelogs based on commits - https://github.com/lalitkapoor/github-changes

  grunt.registerTask('testing', ['concat:browser', 'concat:qunit', 'express', 'watch']);

  grunt.registerTask('build:prod', ['concat:nodejs', 'concat:browser', 'uglify']);

  grunt.registerTask('deploy:commit', ['build:prod', 'bump-commit']);
  grunt.registerTask('deploy:tests', ['shell:pre_commit_tests', 'bump-only:patch', 'uglify']);
  grunt.registerTask('deploy:patch', ['shell:pre_commit_tests', 'bump-only:patch', 'deploy:commit']);
  grunt.registerTask('deploy:minor', ['shell:pre_commit_tests', 'bump-only:minor', 'deploy:commit']);
  grunt.registerTask('deploy:major', ['shell:pre_commit_tests', 'bump-only:major', 'deploy:commit']);
  grunt.registerTask('deploy',       ['deploy:patch']);

  grunt.registerTask('default', ['build:prod']);

};