QUnit.module('Adding entries', config);

test('should add an entry with a basic function', function(assert){

    AMD.define('adunare', ['exports'], function(exports){

        var adunare = function adunare(x, y) {return x + y;};

        exports["default"] = adunare;

    });

    var adunare = AMD.require('adunare');
    var amdEntries = AMD.list_entries();

    assert.equal(adunare(1,2), 3, 'adunare(1, 2) should equal 3');
    ok(amdEntries.indexOf('adunare') !== -1, 'basic adunare should exist on registry');
    assert.equal(amdEntries.length, 1, 'registry length should be 1');

});

test('should add multiple entries', function(assert){

    AMD.define('adunare', ['exports'], function(exports){

        var adunare = function adunare(x, y) {return x + y;};

        exports["default"] = adunare;

    });

    AMD.define('scadere', ['exports'], function(exports){

        var scadere = function scadere(x, y) {
         return x - y;
        };

        exports["default"] = scadere;

    });

    AMD.define('number', ['exports'], function(exports){

        var result = 47;

        exports["default"] = result;

    });


    var adunare = AMD.require('adunare');
    var scadere = AMD.require('scadere');
    var number = AMD.require('number');
    var amdEntries = AMD.list_entries();

    assert.equal(adunare(1,2), 3, 'adunare(1, 2) should equal 3');
    ok(amdEntries.indexOf('adunare') !== -1, 'multiple adunare should exist on registry');

    assert.equal(scadere(10,2), 8, 'scadere(10, 2) should equal 8');
    ok(amdEntries.indexOf('scadere') !== -1, 'scadere should exist on registry');

    assert.equal(number, 47, 'primitive value is saved correctly');
    ok(amdEntries.indexOf('number') !== -1, 'primitive value should exist on registry');

    assert.equal(amdEntries.length, 3, 'registry length should be 3');

});

test('should not add duplicated module names', function(assert){

    AMD.define('scadere', ['exports'], function(exports){

        var scadere = function scadere(x, y) {
            return x - y;
        };

        exports["default"] = scadere;

    });

    assert.throws(
        function(){
            AMD.define('scadere', function(){});
        },
        /duplicatedModule/,
        'should throw on duplicated Module name'
    );

});