QUnit.module('Config', config);

test('should get default config', function(assert){

    var config = AMD.config();
    var defaultConfig = {
        relativePaths: true,
        defaultFile: 'main',
        paths: null
    };

    assert.deepEqual(config, defaultConfig, 'when was not set in the past');

});

test('should get property', function(assert){

    var config = AMD.config('relativePaths');

    assert.equal(config, true, 'when is called with a string');

});

test('should set config', function(assert){

    var customConfig = {
        defaultFile: 'index'
    };
    var expectedConfig = {
        relativePaths: true,
        defaultFile: 'index',
        paths: null
    };

    AMD.config(customConfig);

    var config = AMD.config();

    assert.deepEqual(config, expectedConfig, 'when is called with an object');

});

test('should set config repeatable', function(assert){

    var customConfig = {
        defaultFile: 'index'
    };
    var secondConfig = {
    	relativePaths: false
    };
    var expectedConfig = {
        relativePaths: false,
        defaultFile: 'main',
        paths: null
    };

    AMD.config(customConfig);
    AMD.config(secondConfig);

    var config = AMD.config();

    assert.deepEqual(config, expectedConfig, 'when is called multiple times');

});

test('should set default config', function(assert){

    var expectedConfig = {
        relativePaths: true,
        defaultFile: 'main',
        paths: null
    };

    AMD.config({});

    var config = AMD.config();

    assert.deepEqual(config, expectedConfig, 'when is called with an empty object');

});

test('should set nested config', function(assert){

    var customConfig = {
    	relativePaths: false,
        paths: {
        	prefix:'folder',
        	ignore:['tests']
        }
    };
    var expectedConfig = {
        relativePaths: false,
        defaultFile: 'main',
        paths: {
        	prefix:'folder',
        	ignore:['tests']
        }
    };

    AMD.config(customConfig);

    var config = AMD.config();

    assert.deepEqual(config, expectedConfig, 'when is called with deep objects');

});

