QUnit.module('Prefix paths', config);

test('get paths with prefix', function(assert){

    AMD.config({
        paths: {
            prefix:'pkg'
        }
    });

    AMD.define('pkg/foo', ['exports'], function(exports){

        var result = 'aaa';

        exports["default"] = result;

    });

    AMD.define('pkg/pkg/foo', ['exports'], function(exports){

        var result = 'bbb';

        exports["default"] = result;

    });

    var aaa = AMD.require('foo');
    var bbb = AMD.require('pkg/foo');

    assert.equal(aaa, 'aaa', 'prefix is added');
    assert.equal(bbb, 'bbb', 'prefix is added on paths with duplicated');

});

test('prefix can have ending slash', function(assert){

    AMD.config({
        paths: {
            prefix:'pkgs/'
        }
    });

    AMD.define('pkgs/foo', ['exports'], function(exports){

        var result = 'aaa';

        exports["default"] = result;

    });

    AMD.define('pkgs/pkgs/foo', ['exports'], function(exports){

        var result = 'bbb';

        exports["default"] = result;

    });

    var aaa = AMD.require('foo');
    var bbb = AMD.require('pkgs/foo');

    assert.equal(aaa, 'aaa', 'prefix is added with ending slash');
    assert.equal(bbb, 'bbb', 'prefix is added with ending slash on nested packages');

});

test('ignore string for not prefixing paths', function(assert){

    AMD.config({
        paths: {
            prefix:'packages',
            ignore: 'tests'
        }
    });

    AMD.define('packages/foo', ['exports'], function(exports){

        var result = 'aaa';

        exports["default"] = result;

    });

    AMD.define('tests/foo/string', ['exports'], function(exports){

        var result = 'bbb';

        exports["default"] = result;

    });

    var aaa = AMD.require('foo');
    var bbb = AMD.require('tests/foo/string');

    assert.equal(aaa, 'aaa', 'prefix is set');
    assert.equal(bbb, 'bbb', 'prefix is ignored on specified paths');

});

test('ignore array for not prefixing paths', function(assert){

    AMD.config({
        paths: {
            prefix:'packages',
            ignore: [
                'tests',
                'modules'
            ]
        }
    });

    AMD.define('packages/foo', ['exports'], function(exports){

        var result = 'aaa';

        exports["default"] = result;

    });

    AMD.define('tests/xyz', ['exports'], function(exports){

        var result = 'bbb';

        exports["default"] = result;

    });

    AMD.define('modules/boo', ['exports'], function(exports){

        var result = 'ccc';

        exports["default"] = result;

    });

    var aaa = AMD.require('foo');
    var bbb = AMD.require('tests/xyz');
    var ccc = AMD.require('modules/boo');

    assert.equal(aaa, 'aaa', 'prefix is set');
    assert.equal(bbb, 'bbb', 'prefix is ignored on paths array');
    assert.equal(ccc, 'ccc', 'prefix is ignored on specified paths');

});

test('ignore strings can contain ending slash', function(assert){

    AMD.config({
        paths: {
            prefix:'pachete',
            ignore: [
                'tests/'
            ]
        }
    });

    AMD.define('pachete/foo', ['exports'], function(exports){

        var result = 'aaa';

        exports["default"] = result;

    });

    AMD.define('tests/foo', ['exports'], function(exports){

        var result = 'bbb';

        exports["default"] = result;

    });

    var aaa = AMD.require('foo');
    var bbb = AMD.require('tests/foo');

    assert.equal(aaa, 'aaa', 'prefix is set');
    assert.equal(bbb, 'bbb', 'ending slah is accepted');

});

test('ignore strings should not contain regex special chars', function(assert){

    AMD.config({
        paths: {
            prefix:'folders',
            ignore: [
                'tests.folders/'
            ]
        }
    });

    AMD.define('tests/folders/dep', ['exports'], function(exports){

        var result = 'bbb';

        exports["default"] = result;

    });

    assert.throws(
        function(){
            var result = AMD.require('tests/folders/dep');
        },
        /Ignored prefixes contains invalid chars/,
        'Regex special chars are not allowed in strings'
    );
});


