
QUnit.module('Relative paths', config);

test('should accept relative paths on current directory', function(assert){

    AMD.define('package/parinte', ['exports', './childs/first'], function(exports, dep1){

        var parinte = dep1['default'];

        exports["default"] = parinte;

    });

    AMD.define('package/childs/first', ['exports'], function(exports){

        var firstChild = 'a';

        exports["default"] = firstChild;

    });

    var result = AMD.require('package/parinte');

    assert.deepEqual(result, 'a', 'get correctly the dependency with relativePath in current directory');

});

test('should accept relative paths on parent directory', function(assert){

    AMD.define('package/parinte', ['exports', '../childs/first'], function(exports, dep1){

        var parinte = dep1['default'];

        exports["default"] = parinte;

    });

    AMD.define('childs/first', ['exports'], function(exports){

        var firstChild = 'a';

        exports["default"] = firstChild;

    });

    var result = AMD.require('package/parinte');

    assert.deepEqual(result, 'a', 'get correctly the dependency with relativePath in current directory');

});

test('should accept relative paths on siblings', function(assert){

    AMD.define('foo/a', ['exports', './b'], function(exports, dep1){

        var parinte = dep1['default'];

        exports["default"] = parinte;

    });

    AMD.define('foo/b', ['exports', './c'], function(exports, dep1){

        var firstChild = dep1['default'];

        exports["default"] = firstChild;

    });

    AMD.define('foo/c', ['exports'], function(exports){

        var firstChild = 'c';

        exports["default"] = firstChild;

    });

    var result = AMD.require('foo/a');

    assert.deepEqual(result, 'c', 'get correctly siblings');

});

test('should not accept relative paths based on current directory', function(assert){

    AMD.config({relativePaths: false});

    AMD.define('package/parinte', ['exports', './childs/first'], function(exports, dep1){

        var parinte = dep1['default'];

        exports["default"] = parinte;

    });

    AMD.define('package/parinte/childs/first', ['exports'], function(exports){

        var firstChild = 'a';

        exports["default"] = firstChild;

    });

    assert.throws(
        function(){
            var result = AMD.require('package/parinte');
        },
        /Relative paths are not allowed/,
        'should throw when relative paths are not allowed'
    );

});

test('should not accept relative paths based on parent directory', function(assert){

    AMD.config({relativePaths: false});

    AMD.define('package/parinte', ['exports', '../childs/first'], function(exports, dep1){

        var parinte = dep1['default'];

        exports["default"] = parinte;

    });

    AMD.define('package/childs/first', ['exports'], function(exports){

        var firstChild = 'a';

        exports["default"] = firstChild;

    });

    assert.throws(
        function(){
            var result = AMD.require('package/parinte');
        },
        /Relative paths are not allowed/,
        'should throw when relative paths are not allowed'
    );

});

test('throws when accessing parent module of root', function(assert){

    AMD.define('root', ['exports', '../ychilds'], function(exports, dep1){

        var parinte = dep1['default'];

        exports["default"] = parinte;

    });

    AMD.define('root/a', ['exports', '../../ychilds'], function(exports, dep1){

        var parinte = dep1['default'];

        exports["default"] = parinte;

    });

    assert.equal(AMD.list_entries().length, 2, 'registry length should be 2');

    assert.throws(
        function(){
            var result = AMD.require('root/a');
        },
        /Could not find module/,
        'should throw when access relative higher than root'
    );

    assert.throws(
        function(){
            var result = AMD.require('root');
        },
        /Could not find module/,
        'should throw when access relative bigger than parent and root'
    );

});

